SystemOrganization addCategory: #'Zinc-HTTP-Methods'!
SystemOrganization addCategory: #'Zinc-HTTP-Network'!
SystemOrganization addCategory: #'Zinc-HTTP-Session'!
SystemOrganization addCategory: #'Zinc-HTTP-Core'!

Object subclass: #ZnAuthentication
	instanceVariableNames: 'realm url user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnAuthentication class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:39'!
fromType: aType
	self allSubclassesDo: [:sub | 
		sub type = aType ifTrue: [ ^sub new ]
	].
	^ nil! !

!ZnAuthentication class methodsFor: 'as yet unclassified' stamp: 'cc 5/10/2004 22:16'!
isValidType: aType
	^ self allSubclasses anySatisfy: [:sub | aType = sub type].! !

!ZnAuthentication class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:36'!
type
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:51'!
< aToken
	"Implemented for selecting the most specific token"
	^ self url path size < aToken url path size! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:54'!
acceptUrl: aUrl
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 22:08'!
fromPending: pendingAuth info: infoDict
	"Sets the info of this authentication token to that of the pending token
	and the information dictionary"
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:51'!
max: aToken
	"Implemented for selecting the most specific token"
	^ self < aToken 
		ifTrue: [aToken]
		ifFalse: [self]! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:45'!
password
	^ password! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
password: aPassword
	password := aPassword! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:31'!
realm
	^ realm! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
realm: aRealm
	realm := aRealm.! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 20:14'!
renderHeaderOn: aWriteStream forMethod: aMethod
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 03:35'!
updateAuthInfo: infoDict
	"Update yourself from the authorization-info header that was sent."
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 18:45'!
updateFromInfo: infoDict
	"Update yourself from the information dictionary that was just sent"
	self subclassResponsibility! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:48'!
url
	^ url! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
url: aUrl
	url := aUrl! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:44'!
user
	^ user! !

!ZnAuthentication methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
user: aUser
	user := aUser! !

ZnAuthentication subclass: #ZnBasicAuthentication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnBasicAuthentication class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:36'!
type
	^ 'Basic'! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 20:03'!
acceptUrl: aUrl
	^ aUrl pathstring includesSubString: self url pathstring! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:45'!
encodedUserPass
	^ (Base64MimeConverter mimeEncode: (self user, ':', self password) readStream)! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/8/2004 00:12'!
fromPending: authToken info: infoDict
	self
		url: authToken url;
		user: authToken user;
		password: authToken password;
		realm: (infoDict at: 'realm' ifAbsent: [nil]).! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 20:14'!
renderHeaderOn: aWriteStream forMethod: aMethod
	aWriteStream 
		nextPutAll: self class type;
		nextPutAll: ' ';
		nextPutAll: self encodedUserPass.
	! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 03:35'!
updateAuthInfo: infoDict
	self shouldNotImplement! !

!ZnBasicAuthentication methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 04:42'!
updateFromInfo: infoDict
	self realm: (infoDict at: 'realm' ifAbsent: [^nil]).! !

ZnAuthentication subclass: #ZnDigestAuthentication
	instanceVariableNames: 'method uri domain cnonce nonce nonceCount opaque algorithm qop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnDigestAuthentication class methodsFor: 'as yet unclassified' stamp: 'cc 5/10/2004 00:00'!
type
	^ 'Digest'! !

!ZnDigestAuthentication methodsFor: 'auth accessing' stamp: 'cc 5/9/2004 19:22'!
acceptUrl: aUrl
	^ self domain anySatisfy: [:dom | 
		aUrl pathstring includesSubString: dom pathstring]! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
algorithm
	^ algorithm ifNil: [ algorithm := 'MD5' ]! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
algorithm: anAlgorithm
	| supported |
	supported := #('MD5' 'MD5-sess').

	(supported includes: anAlgorithm) 
		ifTrue: [algorithm := anAlgorithm].! !

!ZnDigestAuthentication methodsFor: 'authentication info' stamp: 'lr 8/30/2010 11:22'!
authorizeRspauth: rspauth qop: aQop
	| a2 |
	a2 := ':', self uri.
	aQop = 'auth-int' ifTrue: [a2 := a2, (self hash: self method entityBody)].

	^ (self requestDigestA1: self getA1 A2: a2) = rspauth
	! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
cnonce
	| from to |
	^ cnonce ifNil: [
		from := self hash: (self nonce, ':', TimeStamp current asSeconds asString).
		to := WriteStream on: String new.
		8 timesRepeat: [to nextPut: from atRandom].
		cnonce := to contents asLowercase.
	]! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
cnonceHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header
		nextPutAll: 'cnonce=';
		nextPutAll: self cnonce quoted.
	
	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/9/2004 19:51'!
commaLine: header
	header nextPut: $,; nextPutAll: String crlf.! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
domain
	^ domain! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
domain: domainCollection
	domain := domainCollection! !

!ZnDigestAuthentication methodsFor: 'auth accessing' stamp: 'cc 5/10/2004 00:15'!
fromPending: authToken info: infoDict
	self
		url: authToken url;
		user: authToken user;
		password: authToken password;
		realm: (infoDict at: 'realm' ifAbsent: [^nil]).

	self
		parseDomains: infoDict;
		nonce: (infoDict at: 'nonce' ifAbsent: [^nil]);
		opaque: (infoDict at: 'opaque' ifAbsent: [nil]);
		algorithm: (infoDict at: 'algorithm' ifAbsent: [nil]);
		qop: (infoDict at: 'qop' ifAbsent: [nil]).! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
getA1
	| a1 |
	a1 := self user, ':', self realm, ':', self password.
	self algorithm = 'MD5-sess'
		ifTrue: [a1 := (self hash: a1), ':', self nonce, ':', self cnonce].

	^ a1 ! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
getA2
	| a2 |
	a2 := self method methodName, ':', self uri.
	self qop = 'auth-int' 
		ifTrue: [a2 := a2, (self method entityBody)].
	
	^ a2! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/11/2004 04:27'!
getRequestDigest
	^ self requestDigestA1: self getA1 A2: self getA2! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/9/2004 21:00'!
hash: aString
	^ (MD5 hashMessage: aString) asString asHex asLowercase! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 20:15'!
method
	^ method! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
method: aMethodName
	method := aMethodName! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
nonce
	^ nonce! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
nonce: aNonce
	nonce := aNonce! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
nonceCount
	nonceCount ifNil: [nonceCount := 0].
	^ nonceCount := nonceCount + 1! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 21:02'!
nonceCountHeader
	^ (self nonceCount hex8 copyFrom: 4 to: 11) asLowercase! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
nonceCountHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header 
		nextPutAll: 'nc=';
		nextPutAll: self nonceCountHeader.

	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
nonceHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'nonce=';
		nextPutAll: self nonce quoted.
	
	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
opaque
	^ opaque! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
opaque: anOpaqueString
	opaque := anOpaqueString! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:26'!
opaqueHeaderOn: header
	self opaque ifNil: [^nil].
	self tabsOn: header.
	header
		nextPutAll: 'opaque=';
		nextPutAll: self opaque quoted.! !

!ZnDigestAuthentication methodsFor: 'auth accessing' stamp: 'lr 8/30/2010 11:22'!
parseDomains: infoDict
	| domainString domains |
	domainString := infoDict at: 'domain' ifAbsent: [domain := Bag with: self url].
	domains := domainString findTokens: ' '.
	
	self domain: (domains collect: [:dom | self url newFromRelativeText: dom withoutQuoting]).! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:06'!
qop
	^ qop! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
qop: someQoP
	| supported |
	supported := #('auth' 'auth-int').

	(supported includes: someQoP) 
		ifTrue: [qop := someQoP].! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
qopHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header 
		nextPutAll: 'qop=';
		nextPutAll: self qop.

	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
realmHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'realm=';
		nextPutAll: self realm quoted.
	
	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'auth accessing' stamp: 'cc 5/10/2004 00:23'!
renderHeaderOn: header forMethod: aMethod
	self uri: aMethod url pathstring.
	self method: aMethod.

	header nextPutAll: self class type, ' '.
	self userHeaderOn: header.
	
	self 
		realmHeaderOn: header;
		nonceHeaderOn: header;
		uriHeaderOn: header;
		qopHeaderOn: header;
		cnonceHeaderOn: header;
		nonceCountHeaderOn: header;
		responseHeaderOn: header;
		opaqueHeaderOn: header.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
requestDigestA1: a1 A2: a2
	| toHash ha1 ha2 |
	ha1 := self hash: a1.
	ha2 := self hash: a2.
	self qop 
		ifNil: [ toHash := ha1, ':', (self nonce, ':', ha2) ]
		ifNotNil: [toHash := ha1, ':', (self nonce, ':', self nonceCountHeader, ':', self cnonce, ':', self qop, ':', ha2) ].
	
	^ self hash: toHash.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/11/2004 04:27'!
responseHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'response=';
		nextPutAll: self getRequestDigest quoted.
	self opaque ifNotNil: [ self commaLine: header ].! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
tabsOn: aWriteStream
	aWriteStream nextPutAll: '        '.! !

!ZnDigestAuthentication methodsFor: 'authentication info' stamp: 'lr 8/30/2010 11:22'!
updateAuthInfo: infoDict
	"Follows RFC 2617 3.2.3 closely."
	| rspqop rspauth rspcnonce rspnc |
	rspqop := infoDict at: 'qop' ifAbsent: [nil].
	rspqop 
		ifNil: [ 
			rspauth := infoDict at: 'rspauth' ifAbsent: [nil].
			rspauth ifNotNil: [ (self authorizeRspauth: rspauth qop: nil) ifFalse: [^nil] ].
			self nonce: (infoDict at: 'nextnonce' ifAbsent: [^nil]) ]
		ifNotNil: [
			rspqop = self qop ifFalse: [^nil].
			rspauth := infoDict at: 'rspauth' ifAbsent: [^nil].
			rspcnonce := infoDict at: 'cnonce' ifAbsent: [^nil].
			rspnc := infoDict at: 'nc' ifAbsent: [^nil].

			rspcnonce = self cnonce ifFalse: [^nil].
			rspnc = self nonceCountHeader ifFalse: [^nil].
			
			(self authorizeRspauth: rspauth)	 ifTrue: [
				self nonce: (infoDict at: 'nextnonce' ifAbsent: [^nil])].
			]! !

!ZnDigestAuthentication methodsFor: 'auth accessing' stamp: 'cc 5/11/2004 04:39'!
updateFromInfo: infoDict
	self
		parseDomains: infoDict;
		nonce: (infoDict at: 'nonce' ifAbsent: [^nil]);
		opaque: (infoDict at: 'opaque' ifAbsent: [nil]);
		algorithm: (infoDict at: 'algorithm' ifAbsent: [nil]);
		qop: (infoDict at: 'qop' ifAbsent: [nil]).! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:20'!
uri
	^ uri! !

!ZnDigestAuthentication methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
uri: digestUri
	uri := digestUri! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:30'!
uriHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'uri=';
		nextPutAll: self uri quoted.

	self commaLine: header.! !

!ZnDigestAuthentication methodsFor: 'header' stamp: 'cc 5/10/2004 00:17'!
userHeaderOn: header
	header
		nextPutAll: 'username=';
		nextPutAll: self user quoted.

	self commaLine: header.! !

Object subclass: #ZnAuthenticationManager
	instanceVariableNames: 'tokens pending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnAuthenticationManager class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:49'!
new
	^ super new initialize! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'SvenVanCaekenberghe 9/1/2010 17:17'!
addPendingUrl: aUrl user: user password: password
	self pending add: (ZnAuthentication new 
			user: user; 
			password: password;
			url: aUrl;
			yourself)! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'cc 5/7/2004 19:50'!
addToken: anAuthToken
	self tokens add: anAuthToken! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
authHeaderForMethod: aMethod on: aStream
	| authtoken |
	authtoken := self selectForMethod: aMethod ifEmpty: [^nil].
	
	aStream nextPutAll: 'Authorization: '.
	authtoken renderHeaderOn: aStream forMethod: aMethod.
	aStream nextPutAll: String crlf.! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'cc 5/11/2004 03:39'!
distributeDigestInfo: infoDict from: aResponse
	(self tokensForUrl: aResponse url) do: [:token | token updateAuthInfo: infoDict].! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'SvenVanCaekenberghe 9/1/2010 17:17'!
distributeInfo: infoDict from: response type: authType
	"If there is a pending authentication token, make it happen. Otherwise,
	it might be data for a current token."
	(ZnAuthentication isValidType: authType) ifFalse: [^nil].

	self updatePendingFor: response url type: authType info: infoDict.
	self updateCurrentFor: response url info: infoDict.
	! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'lr 8/30/2010 11:22'!
getInfoDictionary: rawHeader
	| infodict |
	infodict := Dictionary new.

	(rawHeader findTokens: $,) do: [:token |
		(token findTokens: $=) pairsDo: [:name :value |
			infodict 
				at: name withBlanksTrimmed 
				put: value withBlanksTrimmed withoutQuoting
		]
	].

	^ infodict! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'cc 5/15/2004 09:28'!
getPendingAuthsFor: aUrl
	^ self pending select: [:ptoken | ptoken url = aUrl]! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:39'!
infoFrom: aHeader
	^ (self typeInfoFrom: aHeader) last! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
initialize
	tokens := Bag new.
	pending := Bag new.! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'lr 8/30/2010 11:22'!
parseInfoFromResponse: aResponse
	| authtype info |
	aResponse withHeader: 'www-authenticate' do: [:authheader |
		authtype := self typeFrom: authheader.
		info := self getInfoDictionary: (self infoFrom: authheader).

		self distributeInfo: info from: aResponse type: authtype.
	].

	aResponse withHeader: 'authentication-info' do: [:authheader |
		info := self getInfoDictionary: authheader.
		self distributeDigestInfo: info from: aResponse.
	].! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/7/2004 22:02'!
pending
	^ pending! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/15/2004 11:04'!
selectForMethod: aMethod ifEmpty: emptyBlock
	"Answers the most specific authentication token for the given method and user."
	^ self 
		selectForUrl: aMethod url 
		user: aMethod user 
		ifEmpty: emptyBlock.! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'SvenVanCaekenberghe 9/1/2010 21:00'!
selectForUrl: aUrl user: user ifEmpty: emptyBlock
	"Answers the most specific authentication token for the given url."
	
	| accepted |
	accepted := (self tokensForUrl: aUrl) 
		select: [ :auth | auth user = user ].
	accepted isEmpty ifTrue: [ emptyBlock value ].
	^ accepted max! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/7/2004 19:49'!
tokens
	^ tokens! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/10/2004 23:15'!
tokensForUrl: aUrl
	^ self tokens select: [:auth | auth acceptUrl: aUrl].! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:39'!
typeFrom: aHeader
	^ (self typeInfoFrom: aHeader) first! !

!ZnAuthenticationManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:38'!
typeInfoFrom: aHeader
	^ aHeader findFirstToken: ' '! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'cc 5/10/2004 23:17'!
updateCurrentFor: aUrl info: infoDict
	(self tokensForUrl: aUrl) do: [:token |
		token updateFromInfo: infoDict.
	].! !

!ZnAuthenticationManager methodsFor: 'updating' stamp: 'SvenVanCaekenberghe 9/1/2010 17:17'!
updatePendingFor: aUrl type: authType info: infoDict
	(self getPendingAuthsFor: aUrl) do: [:token || newauth |
		newauth := ZnAuthentication fromType: authType.
		newauth fromPending: token info: infoDict.
		self addToken: newauth.
	].! !

Object subclass: #ZnChunkedStream
	instanceVariableNames: 'socketstream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Network'!

!ZnChunkedStream class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 15:25'!
on: anSS
	^ self new socketstream: anSS; yourself.! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 15:26'!
contents
	^ self upToEnd! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:55'!
contents: aLength
	"HTTP/1.1 spec says we must ignore this"
	^ self upToEnd! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 01:14'!
doesNotUnderstand: message
	^self socketstream 
		perform: message selector 
		withArguments: message arguments! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:28'!
getChunkSize
	^ Number readFrom: (self socketstream upToAll: String crlf) asUppercase base: 16! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 22:08'!
ifDisconnected: aBlock
	self socketstream isConnected ifFalse: aBlock! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:33'!
skip: num
	self socketstream next: num! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 14:49'!
socketstream
	^socketstream! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
socketstream: anSS
	socketstream := anSS! !

!ZnChunkedStream methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
upToEnd
	"Goes to the very end."
	| body chunksize |
	body := WriteStream on: String new.
	
	[(chunksize := self getChunkSize) = 0] whileFalse: [
		self ifDisconnected: [^nil].
		body nextPutAll: (self socketstream next: chunksize).
		self skip: 2.
	].

	^body contents! !

Object subclass: #ZnCookie
	instanceVariableNames: 'attributes url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
addAttribute: name value: value
	| realname |
	realname := name.
	name = 'expires' ifTrue: [realname := 'max-age'].
	self attributes at: realname put: value.! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
attributes
	^ attributes ifNil: [ attributes := Dictionary new ]! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:19'!
domain
	^ self attributes at: 'domain' ifAbsent: [self url authority]! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:06'!
expires
	"Backwards-compatibility"
	^ self maxage! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
maxage
	^ self attributes at: 'max-age' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
maxageTimeStamp
	self expires ifNil: [^ TimeStamp current plusDays: 1].
	^ TimeStamp readFrom: (self expires fromFirstDigit) readStream! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
name
	^ self attributes at: 'name'! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
path
	^ self attributes at: 'path' ifAbsent: [ '\' ].! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
secure
	^ attributes at: 'secure' ifAbsent: [nil]! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:04'!
url
	^url! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
url: aUrl
	url := aUrl! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
value
	^ self attributes at: 'value'! !

!ZnCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:32'!
version
	^ self attributes at: 'version' ifAbsent: [^nil]! !

Object subclass: #ZnCookieJar
	instanceVariableNames: 'cookies tempurl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnCookieJar methodsFor: 'parsing' stamp: 'ctc 11/18/2005 02:31'!
addAttributesFrom: parameters to: aCookie
	| namedparams tokens |
	namedparams := #('max-age' 'expires' 'domain' 'path' 'secure' 'comment').
	tokens := self getTokensFrom: parameters.

	(namedparams includes: tokens first asLowercase) 
	ifTrue: [
		aCookie
			addAttribute: tokens first asLowercase
			value: tokens last] 
	ifFalse: [ 
		aCookie attributes 
			at: 'name' put: tokens first;
			at: 'value' put: tokens last.].! !

!ZnCookieJar methodsFor: 'adding' stamp: 'cc 5/5/2004 06:33'!
addCookie: aCookie
	"Check for expiration date"
	(self checkExpirationForAndRemove: aCookie) ifTrue: [^nil].
	"If we wish to delete a cookie..."
	(self checkForDeletion: aCookie) ifTrue: [^nil].
	"Overwriting cookies"
	self checkForOverride: aCookie.
	self cookies add: aCookie! !

!ZnCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 06:33'!
checkExpirationForAndRemove: aCookie
	(self checkForExpiration: aCookie) ifTrue: [ self removeCookie: aCookie. ^true ].
	^false! !

!ZnCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 06:30'!
checkExpirations
	self cookies do: [:cookie | 
		(self checkForExpiration: cookie) ifTrue: [ self removeCookie: cookie ]
	].! !

!ZnCookieJar methodsFor: 'adding' stamp: 'cc 1/16/2005 20:07'!
checkForDeletion: aCookie
	(self checkForExpiration: aCookie) | (aCookie maxage = 0) 
	ifTrue: [ self cookies do: [:cookie | 
		aCookie name = cookie name 
			ifTrue: [ self cookies remove: cookie. ^true ]]].
	^ false! !

!ZnCookieJar methodsFor: 'adding' stamp: 'SvenVanCaekenberghe 9/1/2010 17:13'!
checkForExpiration: aCookie
	^ aCookie maxageTimeStamp < ZnHTTPDateAdapter GMTStamp! !

!ZnCookieJar methodsFor: 'adding' stamp: 'cc 1/17/2005 01:30'!
checkForOverride: aCookie
	self cookies do: [:cookie |
		(aCookie name = cookie name) 
			& (aCookie path = cookie path) 
			& (aCookie domain = cookie domain) 
		ifTrue: [self cookies remove: cookie]
	].! !

!ZnCookieJar methodsFor: 'gathering' stamp: 'lr 8/30/2010 11:22'!
cookieHeaderForUrl: aUrl on: aStream
	| validCookies |
	validCookies := self cookiesForUrl: aUrl.
	validCookies isEmpty ifTrue: [^nil].
	aStream nextPutAll: 'Cookie: '.
	
	validCookies 
		do: [:cookie | aStream nextPutAll: cookie name, '=', cookie value]
		separatedBy: [aStream nextPutAll: '; '].

	aStream nextPutAll: String crlf.
	! !

!ZnCookieJar methodsFor: 'gathering' stamp: 'lr 8/30/2010 11:22'!
cookies
	^ cookies ifNil: [cookies := Bag new]! !

!ZnCookieJar methodsFor: 'gathering' stamp: 'lr 8/30/2010 11:22'!
cookiesForUrl: aUrl
	| root path domainValid pathValid |
	root := aUrl authority.
	path := aUrl page.

	"Check expirations"
	self checkExpirations.

	"Domain"
	domainValid := self cookies select: [:cookie || domain |
		domain := (cookie domain beginsWith: '.') 
			ifTrue: [cookie domain copyFrom: 2 to: cookie domain size]
			ifFalse: [cookie domain].
 		root includesSubString: domain].

	"Path"
	pathValid := domainValid select: [:cookie | path includesSubString: cookie path].

	^ pathValid! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'lr 8/30/2010 11:22'!
getCookieLines: aHeader
	| token |
	token := 'Set-Cookie:'.
	^ aHeader 
		select: [:line | line includesSubString: token]
		thenCollect: [:line | line copyFrom: token size + 1 to: line size]! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'ctc 11/18/2005 02:35'!
getTokensFrom: parameters
	^ (parameters includesSubString: '=')
		ifTrue: [parameters withBlanksTrimmed findFirstToken: #=]
		ifFalse: [
			(OrderedCollection new)
				add: parameters;
				add: '';
				yourself]! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:02'!
newCookieFromLine: aLine
	| params newcookie |
	newcookie _ HCCookie new.
	params _ aLine findTokens: #;.
	
	params do: [:p | self addAttributesFrom: p to: newcookie].
	newcookie url: self tempurl.
	^ newcookie! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'cc 5/13/2004 02:59'!
parseCookie: aCookieLine
	self addCookie: (self newCookieFromLine: aCookieLine).! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'lr 8/30/2010 11:22'!
parseInfoFromResponse: aResponse
	| lines |
	self tempurl: aResponse url.
	lines := self getCookieLines: aResponse rawHeaders.
	lines do: [:line | self parseCookie: line].
	self tempurl: nil.! !

!ZnCookieJar methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:22'!
removeCookie: aCookie
	self cookies remove: aCookie! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:01'!
tempurl
	^tempurl! !

!ZnCookieJar methodsFor: 'parsing' stamp: 'lr 8/30/2010 11:22'!
tempurl: aUrl
	tempurl := aUrl! !

Object subclass: #ZnHTTPClient
	instanceVariableNames: 'connection sessionManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

ZnHTTPClient class
	instanceVariableNames: 'debug'!

!ZnHTTPClient commentStamp: '<historical>' prior: 0!
For examples of my usage, see the class sides of ZnHTTPSocketFacade.

Cookies and authentication will be kept around as long as you have
a reference to me. For multiple requests within the same 'session',
it is then useful to use the same client instance to execute your methods.

Most of my work is done in #executeMethod:.

Any method that is a subclass of ZnHTTPMethod can be instantiated by
sending the #fromUrl: or #fromUrl:user:password: messages to their
class. A few methods have special instantiation messages in their class,
I suggest you check them out in the "Methods" category.

Authentication
-----------------
I support both Basic and Digest authentication. Depending on how you 
need authentication -

#authenticateAndExecute: is the most useful. This calls #authenticate:
and then executes the method twice. See the message comments. You only
have to send this message once for each 'protection space'.

#authenticatePreempt: is used when you know that the server is expecting
Basic authentication, and you have a large document to send. The protocol
for using it is:
client authenticatePreempt: (some method)
client executeMethod: (some method).

Cookies
--------
Cookies are handled automatically by the ZnSessionManager and the ZnCookieJar.!

ZnHTTPClient class
	instanceVariableNames: 'debug'!

!ZnHTTPClient class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/2/2010 12:06'!
currentVersion
	^ '0.1'! !

!ZnHTTPClient class methodsFor: 'debugging' stamp: 'lr 8/30/2010 11:22'!
debug
	^ debug ifNil: [debug := false]! !

!ZnHTTPClient class methodsFor: 'debugging' stamp: 'cc 1/16/2005 19:46'!
ifDebug: aBlock
	self debug ifTrue: aBlock! !

!ZnHTTPClient class methodsFor: 'instance creation' stamp: 'cc 5/7/2004 21:46'!
new
	^ super new initialize! !

!ZnHTTPClient class methodsFor: 'debugging' stamp: 'lr 8/30/2010 11:22'!
silent
	debug := false! !

!ZnHTTPClient class methodsFor: 'debugging' stamp: 'lr 8/30/2010 11:22'!
verbose
	debug := true! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 5/11/2004 06:48'!
authenticate: aMethod
	self sessionManager authenticate: aMethod! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'lr 8/30/2010 11:22'!
authenticateAndExecuteMethod: aMethod
	"Authenticate this method, then execute using two passes. One to get the
	additional information required for an authentication token, the second
	to get the result."
	| auth |
	self authenticate: aMethod.
	auth := self executeMethod: aMethod.
	"A further error occured..."
	auth response status = 401 ifFalse: [ ^auth ].
	
	^ self executeMethod: aMethod.! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 5/11/2004 06:48'!
authenticatePreempt: aMethod
	self sessionManager authenticatePreempt: aMethod! !

!ZnHTTPClient methodsFor: 'connection open/close' stamp: 'SvenVanCaekenberghe 9/2/2010 11:20'!
connectTo: host port: portNumber 
	^connection := ZnConnectionNew connectToHost: host port: portNumber! !

!ZnHTTPClient methodsFor: 'accessing' stamp: 'cc 5/4/2004 05:15'!
connection
	^ connection ifNotNil: [ connection isFresh ifTrue: [ ^ connection ] ].! !

!ZnHTTPClient methodsFor: 'connection open/close' stamp: 'cc 1/16/2005 19:49'!
connectionTo: host port: portNumber
	connection ifNil: [^self connectTo: host port: portNumber].
	connection ifStale: [^self connectTo: host port: portNumber].
	^connection! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 5/11/2004 05:07'!
errorResponse: aResponse from: aMethod stream: stream
	^ aMethod
		response: aResponse;
		instream: stream;
		yourself.! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 9/1/2010 17:11'!
executeMethod: aMethod
	"Execute a method, send the outstream of the method, then send response for processing.
	Todo: https, (delete options trace), connect (https/http), socks, streamlining"
	| httpconn response command |
	httpconn := self connectionTo: aMethod serverName port: aMethod serverPort.
	command := WriteStream on: String new.
	
	"Generate headers"
	aMethod generateHeadersOn: command.
	self sessionHeadersFor: aMethod on: command.
	aMethod specialHeadersOn: command.
	
	"If method has no content length, we need a crlf"
	aMethod noContentLength: [command nextPutAll: String crlf].
	
	httpconn sendCommand: command contents.
	ZnHTTPClient ifDebug: [Transcript show: command contents; cr; cr].

	self sendStreamFrom: aMethod to: httpconn.

	response := self getResponseFrom: httpconn method: aMethod.
	self getSessionInfoFrom: response.
	^ self 
		parseResponse: response 
		from: aMethod 
		with: httpconn
! !

!ZnHTTPClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/2/2010 12:59'!
executeMethodHeadersOnly: aMethod
	"Execute the method with only headers, don't expect a response,
	and don't close the connection."
	| httpconn command |
	httpconn := self connectTo: aMethod serverName port: aMethod serverPort.
	command := WriteStream on: String new.
	
	"Generate headers"
	aMethod generateHeadersOn: command.
	self sessionHeadersFor: aMethod on: command.
	aMethod specialHeadersOn: command.
	
	"If method has no content length, we need a crlf"
	aMethod noContentLength: [command nextPutAll: String crlf].
	
	httpconn sendCommand: command contents.
	ZnHTTPClient ifDebug: [Transcript show: command; cr; cr].
! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 9/1/2010 17:12'!
getResponseFrom: aConnection method: aMethod
	| header response |
	header := aConnection upToAll: String crlfcrlf.
	response := ZnHTTPResponse fromHeader: header url: aMethod url.
	
	"100 Continue - ignore, get the next response"
	response ifStatus: 100 
		do: [^self getResponseFrom: aConnection method: aMethod].
		
	^response! !

!ZnHTTPClient methodsFor: 'reading' stamp: 'cc 5/5/2004 06:12'!
getSessionInfoFrom: aResponse
	self sessionManager addSessionInfoFrom: aResponse! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 1/10/2005 15:33'!
ifChunked: aResponse do: aBlock
	(aResponse header: 'transfer-encoding' equals: 'chunked') ifTrue: aBlock! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 1/10/2005 22:15'!
ifRedirect: aResponse do: aBlock
	^ aResponse ifStatusClass: 3 do: aBlock! !

!ZnHTTPClient methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 9/1/2010 17:16'!
initialize
	sessionManager := ZnSessionManager new.! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'cc 1/10/2005 22:16'!
okToProceedWith: aResponse then: aBlock
	aResponse ifStatusClass: 2 do: aBlock! !

!ZnHTTPClient methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 9/1/2010 17:15'!
parseResponse: response from: aMethod with: aSocketStream
	| delegate stream |
	"Get the 'true' socket stream, if you will (bug found by Brian Murphy-Dye)"
	stream := aSocketStream socketstream.
	self ifChunked: response do: [ stream := ZnChunkedStream on: stream ].

	self okToProceedWith: response then: [
		^ aMethod 
			response: response;
			instream: stream;
			yourself.
	].

	self ifRedirect: response do: [ 
		delegate := self redirect: response from: aMethod.
		^ self 
			parseResponse: delegate response
			from: aMethod
			with: delegate instream
	].

	^ self errorResponse: response from: aMethod stream: stream

	! !

!ZnHTTPClient methodsFor: 'sending' stamp: 'SvenVanCaekenberghe 9/1/2010 17:13'!
redirect: response from: aMethod
	^ self executeMethod: (
		ZnHTTPGetMethod fromUrl: (
			aMethod url newFromRelativeText: (response getHeader: 'location')))! !

!ZnHTTPClient methodsFor: 'sending' stamp: 'SvenVanCaekenberghe 9/1/2010 17:11'!
sendStreamFrom: aMethod to: aConnection
	| chunkSize cnts |
	"Needfully borrowed from HttpResponse>>#pvtWriteContentsOn:"
	chunkSize := 131072.
	cnts := aMethod outstream.
	cnts ifNil: [^nil].

	ZnHTTPClient ifDebug: [Transcript cr; show: cnts contents].
	
	[cnts atEnd] whileFalse: 
		[aConnection nextPutAll: (cnts next: chunkSize); flush].

	aConnection nextPutAll: String crlf; flush.! !

!ZnHTTPClient methodsFor: 'accessing' stamp: 'cc 5/5/2004 03:36'!
sessionHeadersFor: aMethod
	^ self sessionManager sessionHeadersFor: aMethod! !

!ZnHTTPClient methodsFor: 'accessing' stamp: 'cc 5/15/2004 10:34'!
sessionHeadersFor: aMethod on: aStream
	self sessionManager sessionHeadersFor: aMethod on: aStream! !

!ZnHTTPClient methodsFor: 'accessing' stamp: 'cc 5/7/2004 21:45'!
sessionManager
	^ sessionManager! !

Object subclass: #ZnHTTPClientFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHTTPClientFacade commentStamp: '<historical>' prior: 0!
My class methods are similar to those in HTTPClient,
a class that I am aspiring to replace.!

!ZnHTTPClientFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/2/2010 11:40'!
httpGet: url
	^ ZnHTTPSocketFacade httpGet: url! !

!ZnHTTPClientFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/2/2010 11:40'!
httpGetDocument: url
	^ ZnHTTPSocketFacade httpGetDocument: url! !

Object subclass: #ZnHTTPDateAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHTTPDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:19'!
GMTStamp
	^ self localTimeStampToGMT! !

!ZnHTTPDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:17'!
fromGMT
	"Author is in the Pacific time zone, so that's -8 hours from GMT.
	Please set this accordingly."
	^ (-8)! !

!ZnHTTPDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:16'!
localTimeStampToGMT
	"Everything in HTTPClient is stored in GMT, but sometimes you have to compare
	against current system time.
	See #fromGMT and #toGMT"
	^ TimeStamp current plusSeconds: (self toGMT * 3600)! !

!ZnHTTPDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:15'!
toGMT
	^ self fromGMT negated! !

Object subclass: #ZnHTTPMethod
	instanceVariableNames: 'url acceptedMimetypes contents instream outstream response user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

ZnHTTPMethod subclass: #ZnHTTPGetMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Methods'!

!ZnHTTPGetMethod class methodsFor: 'execution' stamp: 'lr 8/30/2010 11:22'!
retrieveContentsFrom: aUrl on: aClient
	| get |
	get := self fromUrl: aUrl.
	aClient executeMethod: get.
	^ get MIMEDocument contents! !

!ZnHTTPGetMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:21'!
methodName
	^ 'GET'! !

ZnHTTPMethod subclass: #ZnHTTPHeadMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Methods'!

!ZnHTTPHeadMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 05:33'!
methodName
	^ 'HEAD'! !

!ZnHTTPMethod class methodsFor: 'instance creation' stamp: 'cc 5/5/2004 13:54'!
fromUrl: aUrl
	^ self new url: aUrl; yourself.! !

!ZnHTTPMethod class methodsFor: 'instance creation' stamp: 'cc 5/7/2004 23:53'!
fromUrl: aUrl user: user password: password
	^ self new 
		url: aUrl; 
		user: user; 
		password: password; 
		yourself.! !

!ZnHTTPMethod class methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:00'!
httpVersion
	^ 'HTTP/1.1'! !

!ZnHTTPMethod class methodsFor: 'instance creation' stamp: 'cc 5/11/2004 06:15'!
new
	^ super new initialize! !

!ZnHTTPMethod class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/2/2010 12:06'!
userAgentString
	^ 'User-Agent: ',
		'ZincHTTPComponents', '-', 
		ZnHTTPClient currentVersion.! !

!ZnHTTPMethod methodsFor: 'httpadapter' stamp: 'SvenVanCaekenberghe 8/31/2010 14:06'!
MIMEDocument
	| type |
	self response withHeader: 'content-type' do: [ :mimetype | type := mimetype ].
	"If the server did not known, try to guess it"
	(type isNil and: [ self url path last notEmpty ]) ifTrue: [
		type := MIMEDocument guessTypeFromName: self url path last ].
	^ MIMEDocument
		contentType: type
		content: self contents
		url: self url! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
acceptedMimetypes
	^ acceptedMimetypes ifNil: [ acceptedMimetypes := Bag with: 'text/html' ]! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/5/2004 02:18'!
addMimetype: aMimetype 
	self acceptedMimetypes add: aMimetype! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
contents
	^contents ifNil: [
		self instream closeAfter: [
			^contents := self response 
				headerExists: 'content-length'
				do: [:length | self instream contents: length asNumber]
				otherwise: [self instream contents]]]
	! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/9/2004 20:35'!
entityBody
	self subclassResponsibility! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:04'!
generateHeadersOn: aStream
	"Generate the non-session headers for this method"
	self requestHeaderOn: aStream.
	self hostHeaderOn: aStream.
	self userAgentHeaderOn: aStream.
	self mimeTypeHeaderOn: aStream.! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 06:32'!
hostHeaderOn: aStream
	aStream nextPutAll: self hostString; nextPutAll: String crlf.! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:48'!
hostString
	^ 'Host: ', self serverName! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:05'!
instream
	^ instream! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
instream: aStream
	instream := aStream! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:20'!
methodName
	self subclassResponsibility! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:08'!
mimeTypeHeaderOn: aStream

	aStream nextPutAll: 'ACCEPT: '.
	self acceptedMimetypes 
		do: [:type | aStream nextPutAll: type]
		separatedBy: [aStream nextPutAll: ' '].

	aStream nextPutAll: String crlf.! !

!ZnHTTPMethod methodsFor: 'httpadapter' stamp: 'cc 1/16/2005 19:44'!
noContentLength: aBlock
	"This method does not have a content length and must do something special,
	usually a crlf"
	aBlock value.! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:07'!
outstream
	^ outstream! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
outstream: aStream
	outstream := aStream! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 20:02'!
page
	^ self url pageAndQuery! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
password
	^ password! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
password: aPassword
	password := aPassword! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 07:05'!
requestHeaderOn: aStream
	aStream
		nextPutAll: self methodName;
		nextPutAll: ' ';
		nextPutAll: self page;
		nextPutAll: ' ';
		nextPutAll: self class httpVersion;
		nextPutAll: String crlf.! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 19:31'!
response
	^ response! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
response: aResponse
	response := aResponse! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:14'!
serverName
	^ self url authority! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:16'!
serverPort
	^ self url port ifNil: [80]! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:34'!
specialHeadersOn: aStream
	^nil! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:02'!
url
	^ url! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
url: aUrl
	url := aUrl! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
user
	^ user! !

!ZnHTTPMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
user: aUser
	user := aUser! !

!ZnHTTPMethod methodsFor: 'private' stamp: 'cc 5/15/2004 06:33'!
userAgentHeaderOn: aStream
	aStream nextPutAll: self class userAgentString; nextPutAll: String crlf.! !

ZnHTTPMethod subclass: #ZnHTTPMultipartPostMethod
	instanceVariableNames: 'boundary documents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Methods'!

!ZnHTTPMultipartPostMethod class methodsFor: 'instance creation' stamp: 'cc 5/15/2004 10:54'!
fromUrl: aUrl args: argDict
	^ (self fromUrl: aUrl) documents: argDict; yourself! !

!ZnHTTPMultipartPostMethod methodsFor: 'private' stamp: 'cc 5/11/2004 05:54'!
addDocument: aMIMEDocument named: name
	self documents at: name put: aMIMEDocument! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 06:17'!
contentLength
	^ (self documents values collect: [:doc | doc contentStream size]) sum asString.! !

!ZnHTTPMultipartPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:13'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:44'!
contentType
	^ 'multipart/form-data'! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:47'!
contentTypeHeader
	^ 'Content-Type: ', self contentType, '; ', 'boundary=', self mimeBoundary! !

!ZnHTTPMultipartPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:15'!
contentTypeHeaderOn: header
	header
		nextPutAll: 'Content-Type: ';
		nextPutAll: self contentType;
		nextPutAll: '; boundary=';
		nextPutAll: self mimeBoundary;
		nextPutAll: String crlf.! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:48'!
documents
	^ documents! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
documents: aDictionary
	documents := aDictionary! !

!ZnHTTPMultipartPostMethod methodsFor: 'initialization' stamp: 'lr 8/30/2010 11:22'!
initialize
	documents := Dictionary new.! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:43'!
methodName
	^ 'POST'! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
mimeBoundary
	^ boundary ifNil: [
		boundary := (MD5 hashMessage: 
			TimeStamp current asSeconds asString) asString asHex asLowercase]! !

!ZnHTTPMultipartPostMethod methodsFor: 'httpadapter' stamp: 'cc 1/16/2005 19:57'!
noContentLength: aBlock
	"Do nothing - we have a content length"! !

!ZnHTTPMultipartPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
outstream
	| ws startBoundary endBoundary |
	ws := WriteStream on: String new.
	startBoundary := '--', self mimeBoundary.
	endBoundary := '--', self mimeBoundary, '--'.

	self documents keysAndValuesDo: [:name :doc |
		ws nextPutAll: startBoundary, String crlf.
		self printName: name document: doc on: ws.
		ws nextPutAll: String crlf.
	].
	ws nextPutAll: endBoundary.
	^ ws contents readStream! !

!ZnHTTPMultipartPostMethod methodsFor: 'private' stamp: 'cc 5/11/2004 07:30'!
printName: name document: document on: writestream
	"Documents must be of MIMEDocument type - or a facade thereof."
	writestream 
		nextPutAll: 'Content-disposition: multipart/form-data; name=';
		nextPutAll: name.
	(document url notNil 
		and: [(document url path last = '') not])
			ifTrue: [writestream nextPutAll: ' filename=', document url path last].
	writestream nextPutAll: String crlf.

	writestream 
		nextPutAll: 'Content-Type: ';
		nextPutAll: document contentType;
		nextPutAll: String crlfcrlf.

	writestream nextPutAll: document contents.! !

!ZnHTTPMultipartPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:24'!
specialHeadersOn: header
	self contentTypeHeaderOn: header.
	self contentLengthHeaderOn: header.! !

ZnHTTPMethod subclass: #ZnHTTPPostMethod
	instanceVariableNames: 'arguments contentType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Methods'!

!ZnHTTPPostMethod class methodsFor: 'instance creation' stamp: 'cc 5/5/2004 00:05'!
fromUrl: aUrl args: argDict
	^ (self fromUrl: aUrl) arguments: argDict; yourself! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 03:08'!
addArgument: name Value: value
	self arguments at: name put: value! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
argString
	"Return the args in a long string, as encoded in a url"
	| argsString |
	argsString := WriteStream on: String new.
	
	self arguments associations do: [ :assoc |
			argsString nextPutAll: assoc key encodeForHTTP.
			argsString nextPut: $=.
			argsString nextPutAll: assoc value encodeForHTTP. ]
		separatedBy: [ argsString nextPut: $& ].

	^ argsString contents! !

!ZnHTTPPostMethod methodsFor: 'private' stamp: 'cc 1/16/2005 19:55'!
argStringHeaderOn: header
	header nextPutAll: self argString; nextPutAll: String crlf.! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
arguments
	^ arguments ifNil: [ arguments := Dictionary new ]! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
arguments: args
	arguments := args! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 05:45'!
contentLength
	^ self argString size printString! !

!ZnHTTPPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:18'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
contentType
	^ contentType ifNil: [ contentType := 'application/x-www-form-urlencoded' ]! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
contentType: aType
	contentType := aType! !

!ZnHTTPPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:19'!
contentTypeHeaderOn: header
	header
		nextPutAll: 'Content-Type: ';
		nextPutAll: self contentType;
		nextPutAll: String crlf.! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'cc 1/16/2005 19:56'!
getArgument: name
	^self arguments at: name ifAbsent: [ nil ]! !

!ZnHTTPPostMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 03:02'!
methodName
	^ 'POST'! !

!ZnHTTPPostMethod methodsFor: 'httpadapter' stamp: 'cc 1/16/2005 19:54'!
noContentLength: aBlock
	"Do nothing, we have a content length"! !

!ZnHTTPPostMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:24'!
specialHeadersOn: header
	
	self contentTypeHeaderOn: header.
	self contentLengthHeaderOn: header.
	header nextPutAll: String crlf.

	self argStringHeaderOn: header.
! !

ZnHTTPMethod subclass: #ZnHTTPPutMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Methods'!

!ZnHTTPPutMethod class methodsFor: 'instance creation' stamp: 'cc 5/15/2004 05:43'!
fromUrl: aUrl stream: aStream user: user password: password
	^ (self fromUrl: aUrl user: user password: password) outstream: aStream; yourself.! !

!ZnHTTPPutMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 06:42'!
contentLength
	^ self outstream size asString! !

!ZnHTTPPutMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:24'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!ZnHTTPPutMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:41'!
methodName
	^ 'PUT'! !

!ZnHTTPPutMethod methodsFor: 'httpadapter' stamp: 'cc 1/16/2005 19:58'!
noContentLength: aBlock
	"Do nothing - there's a content length"! !

!ZnHTTPPutMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:21'!
specialHeadersOn: header
	self contentLengthHeaderOn: header.! !

Object subclass: #ZnHTTPResponse
	instanceVariableNames: 'headers status version rawHeaders url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHTTPResponse class methodsFor: 'instance creation' stamp: 'lr 8/30/2010 11:22'!
foldLines: lines
	| newLines |
	newLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator 
			ifTrue: [newLines at: newLines size  put: (newLines last, line) ]
			ifFalse: [ newLines add: line ] ].

	^ newLines! !

!ZnHTTPResponse class methodsFor: 'utilities' stamp: 'SvenVanCaekenberghe 9/1/2010 17:11'!
fromHeader: aHeader url: aUrl
	| lines newResponse statusLine |
	ZnHTTPClient ifDebug: [Transcript cr; show: aHeader; cr].
	
	newResponse := self fromUrl: aUrl.
	lines := aHeader findTokens: String crlf.
	statusLine := lines first.
	
	newResponse version: (statusLine findTokens: ' ') first.
	newResponse status: (statusLine findTokens: ' ') second asInteger.
	lines := lines copyFrom: 2 to: lines size.
	lines := self foldLines: lines.
	newResponse rawHeaders: lines.
	lines do: [:line || i |
		i := line indexOf: $:.
		i > 0 ifTrue: 
			[newResponse addHeader: (line copyFrom: 1 to: i - 1) asLowercase
				Value: (line copyFrom: i + 1 to: line size) withBlanksTrimmed]].
	^newResponse! !

!ZnHTTPResponse class methodsFor: 'instance creation' stamp: 'cc 5/5/2004 05:58'!
fromUrl: aUrl
	^ self new url: aUrl; yourself.! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/3/2004 07:17'!
addHeader: headerName Value: headerValue
	self headers at: headerName put: headerValue! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/3/2004 07:16'!
getHeader: headerName
	^ self headers at: headerName ifAbsent: [nil]! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/10/2005 15:26'!
header: headerName equals: aValue
	^ (self headers at: headerName ifAbsent: [^false]) = aValue! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/10/2005 22:25'!
headerExists: headerName
	self headers at: headerName ifAbsent: [^false].
	^true! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/20/2005 00:33'!
headerExists: headerName do: aHeaderBlock otherwise: somethingElse
	^(self headerExists: headerName)
		ifTrue: [aHeaderBlock value: (self getHeader: headerName)]
		ifFalse: somethingElse! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
headers
	^ headers ifNil: [ headers := Dictionary new ]! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/10/2005 21:31'!
ifStatus: aNumber do: aBlock
	(self status = aNumber) ifTrue: aBlock! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/10/2005 21:31'!
ifStatusClass: aNumber do: aBlock
	(self statusClass = aNumber) ifTrue: aBlock! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/5/2004 03:29'!
rawHeaders
	^ rawHeaders! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
rawHeaders: aCollection
	rawHeaders := aCollection! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/3/2004 07:14'!
status
	^ status! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
status: aStatus
	status := aStatus! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/3/2004 07:15'!
statusClass
	^ self status // 100! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/5/2004 05:58'!
url
	^url! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
url: aUrl
	url := aUrl! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 5/3/2004 07:18'!
version
	^ version! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
version: anHttpVersion
	version := anHttpVersion! !

!ZnHTTPResponse methodsFor: 'accessing' stamp: 'cc 1/19/2005 23:51'!
withHeader: headerName do: aBlock
	^ self headerExists: headerName do: aBlock otherwise: [nil]! !

Object subclass: #ZnHTTPSocketFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHTTPSocketFacade commentStamp: '<historical>' prior: 0!
My class methods are similar to those in HTTPSocket, 
a class that I am aspiring to replace.

I show examples of how to do use the Zinc HTTP Components as a client.

Of particular interest are the similarities between httpGet, httpPost, httpPut, and httpPostMultipart. 
Zinc HTTP Components allows these simple methods to be up to an order of magnitude (in the case of multipart) shorter in code length.!

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpGet: urlString
	"self httpGet: 'http://homepage.mac.com/svc/Zinc-HTTP-Components/small.html'."
	
	^ self httpGet: urlString accept: '*/*'! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpGet: url accept: mimeType
	| client get |
	client := ZnHTTPClient new.
	get := ZnHTTPGetMethod fromUrl: url asUrl.
	get addMimetype: mimeType.
	client executeMethod: get.
	^ get MIMEDocument! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpGet: url user: user password: password
	| client get |
	client := ZnHTTPClient new.
	get := ZnHTTPGetMethod fromUrl: url asUrl user: user password: password.
	get addMimetype: '*/*'.
	client authenticateAndExecuteMethod: get.
	^ get MIMEDocument! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpGetDocument: url
	^ self httpGet: url accept: 'application/octet-stream'.! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:57'!
httpGif: url
	"self httpGif: 'http://homepage.mac.com/svc/ADayAtTheBeach/calculator.gif'."
	
	| doc stream |
	doc := self httpGet: url asUrl accept: 'image/gif'.
	doc contentType = 'image/gif' ifFalse: [
		self inform: 'Could not get that GIF from the server.'.
		^ ColorForm extent: 20@20 depth: 8 ].
	stream := RWBinaryOrTextStream with: doc contents.
	stream binary; reset.
	^ (GIFReadWriter on: stream) nextImage.! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:57'!
httpJpeg: url
	"self httpJpeg: 'http://caretaker.wolf359.be/sun-fire-x2100.jpg'."
	
	| doc stream |
	doc := self httpGet: url asUrl accept: 'image/jpeg'.
	doc contentType = 'image/jpeg' ifFalse: [
		self inform: 'Could not get that JPEG from the server.'.
		^ ColorForm extent: 20@20 depth: 8 ].
	stream := RWBinaryOrTextStream with: doc contents.
	stream binary; reset.
	^ (JPEGReadWriter on: stream) nextImage.! !

!ZnHTTPSocketFacade class methodsFor: 'get' stamp: 'SvenVanCaekenberghe 9/1/2010 19:58'!
httpPng: url
	"self httpPng: 'http://www.pharo-project.org/images/pharo.png'."
	
	| doc stream |
	doc := self httpGet: url asUrl accept: 'image/png'.
	doc contentType = 'image/png' ifFalse: [
		self inform: 'Could not get that PNG from the server.'.
		^ ColorForm extent: 20@20 depth: 8 ].
	stream := RWBinaryOrTextStream with: doc contents.
	stream binary; reset.
	^ (PNGReadWriter on: stream) nextImage! !

!ZnHTTPSocketFacade class methodsFor: 'post' stamp: 'cc 5/17/2004 02:11'!
httpPost: url args: argsDict accept: mimeType
	^ self httpPostDocument: url args: argsDict accept: mimeType! !

!ZnHTTPSocketFacade class methodsFor: 'post' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpPost: url args: argsDict user: user password: password
	| client post |
	client := ZnHTTPClient new.
	post := ZnHTTPPostMethod 
		fromUrl: url asUrl 
		user: user 
		password: password.
	post arguments: argsDict.
	client authenticateAndExecuteMethod: post.
	^ post MIMEDocument! !

!ZnHTTPSocketFacade class methodsFor: 'post' stamp: 'cc 5/15/2004 06:02'!
httpPostDocument: url args: argsDict
	^ self httpPostDocument: url asUrl args: argsDict accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'post' stamp: 'SvenVanCaekenberghe 9/1/2010 19:33'!
httpPostDocument: url args: argsDict accept: mimeType
	| client post |
	client := ZnHTTPClient new.
	post := ZnHTTPPostMethod fromUrl: url asUrl args: argsDict.
	post addMimetype: mimeType.
	client executeMethod: post.
	^ post MIMEDocument! !

!ZnHTTPSocketFacade class methodsFor: 'post' stamp: 'SvenVanCaekenberghe 9/1/2010 19:34'!
httpPostMultipart: url args: argsDict accept: mimeType
	"argsDict should be a dictionary containing name -> mimeDocument pairs.
	See the comment in MultipartPostMethod>>#printName:document:on:"
	
	| client multi |
	client := ZnHTTPClient new.
	multi := ZnHTTPMultipartPostMethod fromUrl: url asUrl args: argsDict.
	multi addMimetype: mimeType.
	client executeMethod: multi.
	^ multi MIMEDocument! !

!ZnHTTPSocketFacade class methodsFor: 'put' stamp: 'SvenVanCaekenberghe 9/1/2010 19:34'!
httpPut: stream to: url user: user password: password
	"Upload the contents of the stream to a file on the server, we use
	authenticatePreempt to preemptively send basic authentication."
	
	| client put |
	client := ZnHTTPClient new.
	put := ZnHTTPPutMethod fromUrl: url asUrl stream: stream user: user password: password.
	client 
		authenticatePreempt: put;
		executeMethod: put.
	^ put MIMEDocument! !

Object subclass: #ZnSessionManager
	instanceVariableNames: 'cookiejar authmanager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Session'!

!ZnSessionManager class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:44'!
new
	^ super new initialize! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 05:14'!
addLine: aHeader to: aWriteStream
	aHeader = '' ifTrue: [^nil].
	aWriteStream nextPutAll: aHeader; nextPutAll: String crlf.! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:40'!
addSessionInfoFrom: aResponse
	self cookiejar 
		parseInfoFromResponse: aResponse.
	self authmanager
		parseInfoFromResponse: aResponse.! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/8/2004 00:00'!
authenticate: aMethod
	self authmanager 
		addPendingUrl: aMethod url 
		user: aMethod user 
		password: aMethod password.! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 9/2/2010 16:19'!
authenticatePreempt: aMethod
	| newauth |
	newauth := ZnBasicAuthentication new 
		user: aMethod user; 
		password: aMethod password;
		url: aMethod url;
		yourself.
	self authmanager addToken: newauth.! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:37'!
authmanager
	^ authmanager! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:37'!
cookiejar
	^ cookiejar! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 9/1/2010 17:17'!
initialize
	cookiejar := ZnCookieJar new.
	authmanager := ZnAuthenticationManager new.! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
sessionHeadersFor: aMethod
	| header |
	header := WriteStream on: String new.
	
	self cookiejar cookieHeaderForUrl: aMethod url on: header.
	self authmanager authHeaderForMethod: aMethod on: header.

	^ header contents
! !

!ZnSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:03'!
sessionHeadersFor: aMethod on: header
	self cookiejar cookieHeaderForUrl: aMethod url on: header.
	self authmanager authHeaderForMethod: aMethod on: header.! !

SocketStream subclass: #ZnConnectionNew
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Network'!

!ZnConnectionNew commentStamp: '<historical>' prior: 0!
ZnConnectionNew is a plugin replacement for ZnConnection.

The old code was using its own SocketStream, now we're using the system supplied SocketStream.

Some backwards compatibility protocol was added, could be cleaned up later.

It is probably also possible to use SocketStream directly.!

!ZnConnectionNew class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/2/2010 11:19'!
connectToHost: host port: portNumber
	^ self openConnectionToHostNamed: host port: portNumber! !

!ZnConnectionNew methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/2/2010 11:27'!
closeAfter: aBlock
	aBlock ensure: [self close]! !

!ZnConnectionNew methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/2/2010 11:29'!
contents: aLength
	^ self next: aLength! !

!ZnConnectionNew methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/2/2010 11:29'!
socketstream
	^ self! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
page
	| last page |
	last := self path last.
	page := self pathstring.

	(last = '') ifFalse: [page := page, '/', self path last].

	^ page! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'cc 5/7/2004 21:43'!
pageAndQuery
	^ self page, (self query ifNotNil: ['?', self query] ifNil: ['']).! !

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
pathstring
	| ws dirs |
	ws := WriteStream on: String new.
	ws nextPut: $/.
	dirs := self path reject: [ :e | e = self path last ].
	dirs do: [ :pathelem | ws nextPutAll: pathelem ] separatedBy: [ ws nextPut: $/ ].
	(self path last = '' and: [ self path first ~= '' ])
		ifTrue: [ ws nextPut: $/ ].
	^ ws contents! !

Stream subclass: #ZnBufferedStream
	instanceVariableNames: 'buffer startPos endPos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Network'!

!ZnBufferedStream class methodsFor: 'instance creation' stamp: 'cc 5/4/2004 23:27'!
on: aString
	^self basicNew on: aString! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
contents
	"return contents, non-destructive"
	^buffer copyFrom: startPos to: endPos! !

!ZnBufferedStream methodsFor: 'testing' stamp: 'lr 8/30/2010 11:22'!
includes: aChar
	"answer whether buffer includes aChar within bounds"

	| tmp |
	tmp := buffer indexOf: aChar startingAt: startPos.
	^ tmp between: startPos and: endPos! !

!ZnBufferedStream methodsFor: 'testing' stamp: 'lr 8/30/2010 11:22'!
includesAll: aString
	"answer whether the buffer contains aString"

	| tmp |
	tmp := buffer findString: aString startingAt: startPos.
	^ tmp between: startPos and: endPos! !

!ZnBufferedStream methodsFor: 'testing' stamp: 'cc 5/4/2004 23:27'!
isEmpty
	^self size <= 0! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
match: aString
	self size < aString size
		ifTrue: [^false].
	(self next: aString size) = aString
		ifTrue: [^true]
		ifFalse: [self on: aString, self upToEnd].
	^false! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
next
	"return the next character"
	"consume it"
	^buffer at: (startPos := startPos + 1) - 1! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
next: anInteger
	"return next anInteger characters"
	"consumes them"
	^buffer copyFrom: startPos to: (startPos := startPos + anInteger) - 1! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
nextPutAll: aString
	buffer := (buffer copyFrom: startPos to: endPos), aString.
	startPos := 1.
	endPos := buffer size! !

!ZnBufferedStream methodsFor: 'initialize-release' stamp: 'lr 8/30/2010 11:22'!
on: aString
	buffer := aString.
	startPos := 1.
	endPos := buffer size! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
peek
	"return the next character without consuming it"
	"consume it"
	^buffer at: startPos! !

!ZnBufferedStream methodsFor: 'printing' stamp: 'cc 5/4/2004 23:27'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self size asString, ')'! !

!ZnBufferedStream methodsFor: 'initialize-release' stamp: 'cc 5/4/2004 23:27'!
reset
	"hack"
	self on: ''! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
size
	^endPos - startPos + 1! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
upTo: aChar
	"return String up to aChar, but not including"
	"consumes the returned string"
	| i |
	i := buffer indexOf: aChar startingAt: startPos.
	(i <= 0 or: [i > endPos])
		ifTrue: [^self upToEnd].
	^self next: (i-startPos)! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
upToAll: aString
	"return a string before the occurence of aString, if any"
	"consume it"
	| i |
	i := buffer findString: aString startingAt: startPos.
	(i <= 0 or: [i > endPos])
		ifTrue: [^self upToEnd].
	^self next: (i-startPos)! !

!ZnBufferedStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
upToEnd
	"return buffer contents,
	consume all my buffer"
	| string |
	string := self contents.
	self reset.
	^string! !

Stream subclass: #ZnSocketStream
	instanceVariableNames: 'socket inStream outStream inDataCount outDataCount autoFlush binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Network'!

!ZnSocketStream commentStamp: '<historical>' prior: 0!
Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:47

HTTPSocketStream
- some changes to gradually enable client-side work

Comment from kom46:
HTTPSocketStream
- stream interface to Socket
- HttpRequest (and other users) don't know anything about sockets

bolot 12/16/1999 15:01
- re-doing the interface
  - inStream will replace buffer
  - outStream will be the buffer for sending data
  - nextPut:/nextPutAll: will write to outStream
  - flush will be required to send data
- TODO
  - stop using buffer, eventually remove it
!

ZnSocketStream subclass: #ZnConnection
	instanceVariableNames: 'keepalive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Network'!

!ZnConnection commentStamp: 'cc 5/5/2004 00:31' prior: 0!
I contain the HTTP Connection Stream, and also fix some things in SocketStream (rather than changing the class itself).

I do nifty things like KeepAlive.!

!ZnConnection class methodsFor: 'instance creation' stamp: 'cc 5/4/2004 05:31'!
connectToHost: host port: portNumber
	Socket initializeNetwork.
	^ self on: (Socket new connectToHostNamed: host port: portNumber)! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 21:00'!
defaultTimeout
	"300 seconds for keepalive timeout"
	^ 300! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:49'!
ifStale: aBlock
	self isStale ifTrue: aBlock! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:15'!
isFresh
	^ self socket isValid and: [self keepalive < TimeStamp current]! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:15'!
isStale
	^ self isFresh not! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'lr 8/30/2010 11:22'!
keepalive
	^ keepalive ifNil: [ keepalive := TimeStamp current plusSeconds: self defaultTimeout ]! !

!ZnConnection methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 9/2/2010 13:09'!
sendCommand: aCommand
	"Send the command and flush it"
	
	self nextPutAll: aCommand; flush.! !

!ZnSocketStream class methodsFor: 'instance creation' stamp: 'bolot 12/16/1999 18:06'!
on: socket
	"create a socket stream on a server socket"
	"for HTTP style header must call readHeader explicitly"
	^self basicNew initialize: socket! !

!ZnSocketStream methodsFor: 'testing' stamp: 'cc 1/19/2005 21:23'!
atEnd
	^self isConnected not & inStream isEmpty & self isDataAvailable not! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
autoFlushOff
	"Output is sent only when flush method is called"
	autoFlush := false! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
autoFlushOn
	"Have nextPutAll:, cr, crlf flush output buffer"
	autoFlush := true! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 13:35'!
binary
	"Set the stream to read/write binary data"
	self setBinary: true! !

!ZnSocketStream methodsFor: 'private' stamp: 'rew 5/18/2001 16:40'!
bufferClass
	^self isBinary
		ifTrue:[ByteArray]
		ifFalse:[String]! !

!ZnSocketStream methodsFor: 'private' stamp: 'rew 5/18/2001 16:40'!
bufferSized: anInteger
	^self bufferClass new: anInteger! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'bolot 2/5/2001 15:46'!
close
	self flush.
	self socket closeAndDestroy: 30! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'cc 1/19/2005 21:20'!
closeAfter: aBlock
	aBlock ensure: [self close]! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'cc 1/20/2005 00:01'!
contents: aLength
	^ self next: aLength! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'bolot 12/27/1999 12:17'!
cr
	self nextPutAll: String cr! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'bolot 8/11/1999 22:13'!
crlf
	self nextPutAll: String crlf! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'rew 5/21/2001 12:22'!
flush
	socket isOtherEndClosed
		ifFalse: [socket sendData: (outStream contents)].
	"isOtherEndConnected does not exist in 2.8 image"
	"socket isOtherEndConnected
		ifTrue: [socket sendData: outStream contents]."
	self incOutDataCountBy: outStream contents size.
	self resetOutStream! !

!ZnSocketStream methodsFor: 'private' stamp: 'lr 8/30/2010 11:22'!
for: timeLimit do: aBlock every: interval while: booleanBlock
	| end |
	end := TimeStamp current plusSeconds: timeLimit.
	
	[booleanBlock value & (TimeStamp current < end)]
		whileTrue: [ 
			aBlock value.
			(Delay forMilliseconds: interval) wait]! !

!ZnSocketStream methodsFor: 'statistics' stamp: 'bolot 12/16/2000 03:03'!
inBytes "actual number of bytes read by the app"
	^self inDataCount - inStream size! !

!ZnSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:08'!
inDataCount
	^inDataCount! !

!ZnSocketStream methodsFor: 'statistics' stamp: 'lr 8/30/2010 11:22'!
incInDataCountBy: increment
	"increment inDataCount by"
	inDataCount := inDataCount + increment! !

!ZnSocketStream methodsFor: 'statistics' stamp: 'lr 8/30/2010 11:22'!
incOutDataCountBy: increment
	"increment outDataCount by"
	outDataCount := outDataCount + increment! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'rew 5/18/2001 17:03'!
initialize
	self resetStatus.
	self autoFlushOn.
	self text.
	self resetInStream.
	self resetOutStream! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'bolot 12/16/2000 01:46'!
initialize: aSocket
	self initialize.
	self socket: aSocket! !

!ZnSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 17:05'!
isAutoFlushOn
	^autoFlush! !

!ZnSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 17:11'!
isBinary
	^binary! !

!ZnSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:47'!
isConnected
	^self socket isConnected! !

!ZnSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:47'!
isDataAvailable
	^self socket dataAvailable! !

!ZnSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:48'!
isOtherEndConnected
	^self socket isOtherEndConnected! !

!ZnSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 13:23'!
isText
	^self isBinary not! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'rew 5/18/2001 18:09'!
lf
	self nextPutAll: (String with: Character lf)! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
localAddress
	^socket localAddress! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
localPort
	^socket localPort! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'bolot 12/16/2000 03:04'!
match: aString
	"position past the next occurrence of aString"
	"bolot 12/13/2000 00:49 - must begin with it"
	"answer if aString is found"
	inStream size < aString size
		ifTrue: [self flag: #todo. "should try to read in more, unless #atEnd"
			"but since atEnd is somewhat broken, don't use it yet"
			^false].

	(inStream contents beginsWith: aString)
		ifTrue: [inStream next: aString size.
			^true].

	^false! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'bolot 1/4/2000 20:31'!
next
	self atEnd ifTrue: [^nil].
	inStream isEmpty ifTrue: [self pvtGetData].
	^inStream next! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'cc 1/20/2005 00:06'!
next: aLength
	"Wait for data up to aLength, disconnection, or timeout (30 sec)"
	self
		for: 30
		do: [self isDataAvailable ifTrue: [self pvtGetData]]
		every: 200
		while: [self isConnected & inStream size < aLength].

	^inStream next: aLength
! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'bolot 1/5/2000 15:59'!
nextChunk
	"getData front - retrieve all data available now"
	inStream isEmpty
		ifTrue: [self pvtGetData].
	^inStream upToEnd! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'rew 5/19/2001 11:24'!
nextPut: char
	self nextPutAll: (self bufferClass with: char)! !

!ZnSocketStream methodsFor: 'stream out' stamp: 'rew 5/18/2001 17:07'!
nextPutAll: string
	outStream nextPutAll: string.
	self isAutoFlushOn ifTrue: [self flush]! !

!ZnSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:10'!
outDataCount
	^outDataCount! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'lr 8/30/2010 11:22'!
putDataBack: string
	| tempString |
	tempString := inStream contents.
	self resetInStream.
	inStream nextPutAll: string; nextPutAll: tempString! !

!ZnSocketStream methodsFor: 'private' stamp: 'lr 8/30/2010 11:22'!
pvtGetData
	| data buffer bytesRead|
	buffer := self bufferSized: 4000.
	(socket isConnected and: [socket dataAvailable not])
		ifTrue:[socket waitForDataFor: 200].
	bytesRead := socket receiveDataInto: buffer.
	data := buffer
			copyFrom: 1
			to: bytesRead.
	inStream nextPutAll: data.
	self incInDataCountBy: bytesRead.
	^data! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
remoteAddress
	^socket remoteAddress! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:46'!
remotePort
	^socket remotePort! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/1/2010 17:15'!
resetInStream
	inStream := ZnBufferedStream on: (self bufferSized: 0)! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'cc 1/19/2005 21:19'!
resetInStreamAfter: aBlock
	aBlock ensure: [self resetInStream]! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/1/2010 17:15'!
resetOutStream
	outStream := ZnBufferedStream on: (self bufferSized: 0)! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'cc 1/19/2005 21:20'!
resetOutStreamAfter: aBlock
	aBlock ensure: [self resetOutStream]! !

!ZnSocketStream methodsFor: 'initialize-release' stamp: 'lr 8/30/2010 11:22'!
resetStatus
	inDataCount := 0.
	outDataCount := 0! !

!ZnSocketStream methodsFor: 'private' stamp: 'lr 8/30/2010 11:22'!
setBinary: aBoolean
	"Set the stream to handle binary or text"
	binary := aBoolean.
	self resetInStream.
	self resetOutStream! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'bolot 2/5/2001 15:44'!
socket
	^socket! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'lr 8/30/2010 11:22'!
socket: aSocket
	socket := aSocket.
! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'cc 1/10/2005 15:32'!
socketstream
	^self! !

!ZnSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 13:36'!
text
	"Set the stream to read/write text data"
	self setBinary: false! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'lr 8/30/2010 11:22'!
upTo: delim 
	| string |
	(inStream includes: delim)
		ifTrue: ["Found the delimiter part way into buffer"
			string := inStream upTo: delim.
			inStream next.
			^string].
	self isConnected not & self isDataAvailable not
		ifTrue: ["Never found it, and hit end of file"
			string := inStream contents.
			self resetInStream.
			^string].
	"Never found it, but there's more..."
	self pvtGetData.
	^self upTo: delim! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'lr 8/30/2010 11:22'!
upToAll: delims
	| string |
	(inStream includesAll: delims)
		ifTrue: ["Found the delimiter part way into buffer"
			string := inStream upToAll: delims.
			inStream next: delims size.
			^string].
	self isConnected not & self isDataAvailable not
		ifTrue: ["Never found it, and hit end of file"
			^inStream upToEnd].
	"Never found it, but there's more..."
	self pvtGetData.
	^self upToAll: delims! !

!ZnSocketStream methodsFor: 'stream in' stamp: 'cc 1/19/2005 22:05'!
upToEnd
	[self isConnected or: [self isDataAvailable]]
		whileTrue:[self pvtGetData].
	self resetInStreamAfter: [^inStream contents]
! !

!String methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
findFirstToken: aToken
	"Finds the first occurance of aToken, and answeres a collection of everything before and after."
	| index tokens |
	(self includesSubString: aToken) ifFalse: [^ self].
	index := self findString: aToken.
	tokens := OrderedCollection new.

	^ tokens
		add: (self copyFrom: 1 to: index - 1);
		add: (self copyFrom: index + aToken size to: self size);
		yourself.! !

!String methodsFor: '*zinc-http' stamp: 'lr 8/30/2010 11:22'!
fromFirstDigit
	| index |
	index := 1.
	[ (self at: index) isDigit ] whileFalse: [ index := index + 1 ].
	
	^ self copyFrom: index to: self size! !

!String methodsFor: '*zinc-http' stamp: 'cc 5/9/2004 19:28'!
quoted
	"Answers the receiver with a copy of self surrounded by double quotes."
	^ $" asString, self, $" asString! !
